﻿// <copyright file="EntityFrameworkConceptualModelTemplate.cs" company="T4 Toolbox Team">
//  Copyright © T4 Toolbox Team. All Rights Reserved.
// </copyright>

namespace T4Toolbox.EntityFramework
{
    using System;
    using System.Data.Entity.Design;

    /// <summary>
    /// Generates Entity Framework conceptual model.
    /// </summary>
    public class EntityFrameworkConceptualModelTemplate : EntityFrameworkModelTemplate
    {
        /// <summary>
        /// Stores the XML generated by this template.
        /// </summary>
        private string conceptualModel;

        /// <summary>
        /// Gets or sets the entity model generator instance.
        /// </summary>
        /// <value>
        /// An <see cref="EntityModelSchemaGenerator"/> with generated metadata.
        /// </value>
        public EntityModelSchemaGenerator EntityModelSchemaGenerator { get; set; }

        /// <summary>
        /// Gets the Entity Framework conceptual model generated by this template.
        /// </summary>
        /// <value>
        /// A <see cref="String"/> that contains generated CSDL in XML format.
        /// </value>
        public string ConceptualModel
        {
            get { return this.conceptualModel; }
        }

        /// <summary>
        /// Generates conceptual model.
        /// </summary>
        /// <returns>
        /// A <see cref="String"/> with generated XML.
        /// </returns>
        public override string TransformText()
        {
            this.WriteXml(this.EntityModelSchemaGenerator.WriteModelSchema);
            this.conceptualModel = this.GenerationEnvironment.ToString();
            return this.conceptualModel;
        }

        /// <summary>
        /// Validates properties before code generation.
        /// </summary>
        protected override void Validate()
        {
            if (this.EntityModelSchemaGenerator == null)
            {
                throw new TransformationException("EntityModelSchemaGenerator property must be assigned");
            }
        }
    }
}
