﻿// <copyright file="XsdProcessorTest.cs" company="T4 Toolbox Team">
//  Copyright © T4 Toolbox Team. All Rights Reserved.
// </copyright>

namespace T4Toolbox.DirectiveProcessors
{
    using System;
    using System.IO;
    using System.CodeDom.Compiler;
    using System.Collections.Generic;
    using Microsoft.CSharp;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Microsoft.VisualStudio.TextTemplating;

    /// <summary>
    /// This is a test class for <see cref="TransformationContextProcessor"/>
    /// </summary>
    [TestClass]
    public class TransformationContextProcessorTest
    {
        TransformationContextProcessor processor;

        /// <summary>
        /// Initializes environment before each test method
        /// </summary>
        [TestInitialize]
        public void TestInitialize()
        {
            // Create directive processor
            this.processor = new TransformationContextProcessor();

            // Prepare for directive processing
            CodeDomProvider languageProvider = new CSharpCodeProvider();
            string templateContents = "<#@ TransformationContext processor=\"T4Toolbox.TransformationContextProcessor\" #>";
            CompilerErrorCollection errors = new CompilerErrorCollection();
            this.processor.StartProcessingRun(languageProvider, templateContents, errors);
        }

        /// <summary>
        /// Verifies that <see cref="TransformationContextProcessor.GenerateConstructor"/> method generates derived <see cref="TransformationContext"/> class correctly.
        /// </summary>
        [TestMethod]
        public void Test_TransformationContext()
        {
            this.ProcessDirective();
            string classCode = this.processor.GetClassCodeForProcessingRun();
            StringAssert.Contains(classCode, "public abstract class TransformationContext : T4Toolbox.TransformationContext");
            StringAssert.Contains(classCode, "public new static GeneratedTextTransformation Transformation");
            StringAssert.Contains(classCode, "return ((GeneratedTextTransformation)(T4Toolbox.TransformationContext.Transformation));");
        }

        /// <summary>
        /// Verifies that <see cref="TransformationContextProcessor.GenerateConstructor"/> method generates constructor code correctly.
        /// </summary>
        [TestMethod]
        public void Test_GenerateConstructor()
        {
            this.ProcessDirective();
            string classCode = this.processor.GetClassCodeForProcessingRun();
            StringAssert.Contains(classCode, "public GeneratedTextTransformation()");
            StringAssert.Contains(classCode, "TransformationContext.OnTransformationStarted(this);");
        }

        /// <summary>
        /// Verifies that <see cref="TransformationContextProcessor.GenerateDisposeMethod"/> method generates dispose method correctly.
        /// </summary>
        [TestMethod]
        public void Test_GenerateDisposeMethod()
        {
            this.ProcessDirective();
            string classCode = this.processor.GetClassCodeForProcessingRun();
            StringAssert.Contains(classCode, "protected override void Dispose(bool disposing)");
            StringAssert.Contains(classCode, "if (disposing)");
            StringAssert.Contains(classCode, "TransformationContext.OnTransformationEnded(this);");
        }

        private void ProcessDirective()
        {
            this.processor.ProcessDirective("TransformationContext", new Dictionary<string, string>());
        }
    }
}
